# ../scripts/custom/gg_ffa/gg_ffa.py

# =============================================================================
# >> IMPORTS
# =============================================================================
# EventScripts Imports
#   ES
from es import getindexfromhandle
from es import ServerVar

# SPE Imports
from spe import detourFunction
from spe import HookAction
from spe import HookType
from spe import makeObject
from spe import undetourFunction

# SPE Tools Imports
#   Entity
from spe.tools.entity import SPEEntity
#   Player
from spe.tools.player import SPEPlayer

# GunGame Imports
#   Addons
from gungame51.core.addons.shortcuts import AddonInfo

# Script Imports
from modules.flashed import flashed_players
from modules.radar import radar_disabler
from modules.teams import original_teams

# =============================================================================
# >> ADDON REGISTRATION/INFORMATION
# =============================================================================
info = AddonInfo()
info.name = 'gg_ffa'
info.title = 'GG Free For All'
info.author = 'Satoon101'
info.version = '1.4'
info.conflicts = ['gg_teamplay', 'gg_teamwork']

# =============================================================================
# >> PUBLIC VARIABLE
# =============================================================================
ServerVar(info.name + '_version', info.version,
        info.title + ' Version').set(info.version)
ServerVar(info.name + '_version').makepublic()


# =============================================================================
# >> LOAD & UNLOAD
# =============================================================================
def load():
    '''Called when the script is loaded'''

    # Register the OnTakeDamage detour
    detourFunction('OnTakeDamage', HookType.Pre, damage_callback_pre)

    # Start the disable radar loop
    radar_disabler.disable_radar()


def unload():
    '''Called when the script is unloaded'''

    # Unregister the OnTakeDamage detour
    undetourFunction('OnTakeDamage', HookType.Pre, damage_callback_pre)

    # Stop the disable radar loop
    radar_disabler.stop_disabler()


# =============================================================================
# >> DETOUR CALLBACKS
# =============================================================================
def damage_callback_pre(args):
    '''Fired when any player is taking damage'''

    # Get the takedamage instance
    take_damage_info = makeObject('CTakeDamageInfo', args[0])

    # Get the attacker's handle
    attacker_handle = take_damage_info.hAttacker

    # Get the attacker's index
    attacker_index = getindexfromhandle(attacker_handle)

    # Is there an attacker?
    if attacker_index:

        # Get the victim's SPEPlayer instance
        victim = SPEPlayer.get_instance_from_pointer(args[1])

        # Is the damage self damage?
        if attacker_index != victim.index:

            # Get the attacker's SPEEntity instance
            attacker = SPEEntity(attacker_index)

            # Are the players on the same team?
            if victim.team == attacker.team:

                # Add the player to original_teams
                original_teams[victim] = victim.team

                # Switch the victim to the opposite team
                victim.team = 5 - victim.team

    # Return from the detour
    return (HookAction.Continue, 0)


# =============================================================================
# >> GAME EVENTS
# =============================================================================
def player_blind(event_var):
    '''Called when a player is blinded by a flashbang'''

    # Add the player to the flashed_players set
    flashed_players.add(event_var['userid'])


def player_death(event_var):
    '''Called when a player dies'''

    # Remove the player from the flashed_players set
    flashed_players.discard(event_var['userid'])
