# ../scripts/custom/gg_ffa/modules/radar.py

# =============================================================================
# >> IMPORTS
# =============================================================================
# Python Imports
#   Time
from time import time

# EventScripts Imports
#   ES
from es import getUseridList
from es import isbot
from es import setplayerprop
#   Gamethread
from gamethread import cancelDelayed
from gamethread import delayedname

# SPE Tools Imports
#   Player
from spe.tools.player import SPEPlayer

# Script Imports
from flashed import flashed_players


# =============================================================================
# >> CLASSES
# =============================================================================
class _RadarDisabler(object):
    '''Class used to disable all living player's radar'''

    def disable_radar(self):
        '''
        Method used to loop through all living players and disable their radar
        '''

        # Loop through all living non-flashed players
        for userid in self.non_flashed_players:

            # Set the player's properties do disable the radar
            self.set_player_properties(userid)

        # Loop every .5 seconds to continue disabling radar
        delayedname(0.5, 'gg_ffa_radar_main', self.disable_radar)

    def stop_disabler(self):
        '''Method used to stop disabling radar on unload'''

        # Loop through all living non-flashed players
        for userid in self.non_flashed_players:

            # Reset the player's properties to enable the radar
            self.set_player_properties(userid, True)

        # Stop the loop
        cancelDelayed('gg_ffa_radar_main')

    @property
    def non_flashed_players(self):
        '''Generator method used to get living non-flashed players'''

        # Loop through all players on the server
        for userid in getUseridList():

            # Is the player flashed?
            if userid in flashed_players:

                # Don't yield players that are currently flashed
                continue

            # Is the player dead?
            if SPEPlayer(userid).isdead:

                # Cannot disable dead player's radar, so continue
                continue

            # Is the player a bot?
            if isbot(userid):

                # Do not bother removing radar from bots
                continue

            # Yield the player to disable/enable their radar
            yield userid

    @staticmethod
    def set_player_properties(userid, unloading=False):
        '''Method used to disable/enable a player's radar'''

        # Set the duration to be used
        set_time = 0 if unloading else time()

        # Set the player's flash alpha
        setplayerprop(userid, 'CCSPlayer.m_flFlashMaxAlpha', 0)

        # Set the player's flash duration
        setplayerprop(userid, 'CCSPlayer.m_flFlashDuration', set_time)

# Create the instance of _RadarDisabler
radar_disabler = _RadarDisabler()
