# ../scripts/custom/gg_ffa/modules/teams.py

# =============================================================================
# >> IMPORTS
# =============================================================================
# EventScripts Imports
#   Gamethread
from gamethread import queue


# =============================================================================
# >> CLASSES
# =============================================================================
class _OriginalTeams(dict):
    '''Class to hold player teams, so that players are
        always switched back to their original teams'''

    def __setitem__(self, player, team):
        '''
        Adds the player to the dictionary and deletes it after a small delay
        '''

        # Is the player already in the dictionary?
        if not player in self:

            # Add them to the dictionary
            super(_OriginalTeams, self).__setitem__(player, team)

            # Remove them after a small delay
            queue(self.__delitem__, player)

    def __delitem__(self, player):
        '''Sets a player back to their old team
            and removes them from the dictionary'''

        # Is the player in the dictionary?
        if player in self:

            # Set the player back to their original team
            player.team = self[player]

            # Remove the player from the dictionary
            super(_OriginalTeams, self).__delitem__(player)

# Create a dictionary to store player teams
original_teams = _OriginalTeams()
